@@ -23,12 +23,17 @@ SOFTWARE.
 #define MODE_UNKNOWN -1
 #define BOARD        10
 #define BCM          11
+#define SERIAL       40
+#define SPI          41
+#define I2C          42
+#define PWM          43
 
 int gpio_mode;
 const int pin_to_gpio_rev1[27];
 const int pin_to_gpio_rev2[27];
 const int (*pin_to_gpio)[27];
 int gpio_direction[54];
+int revision;
 
 int get_gpio_number(int channel, unsigned int *gpio);
 int setup_error;
